@@ -1,6 +1,6 @@
 /************************************************************************
  * curses_client.c  dopewars client using the (n)curses console library *
- * Copyright (C)  1998-2020  Ben Webb                                   *
+ * Copyright (C)  1998-2022  Ben Webb                                   *
  *                Email: benwebb@users.sf.net                           *
  *                WWW: https://dopewars.sourceforge.io/                 *
  *                                                                      *
@@ -32,6 +32,7 @@
 #endif
 #include <ctype.h>
 #include <signal.h>
+#include <assert.h>
 #include <errno.h>
 #include <glib.h>
 #include "configfile.h"
@@ -77,7 +78,7 @@ static FightPoint fp;
 static void display_intro(void);
 static void ResizeHandle(int sig);
 static void CheckForResize(Player *Play);
-static int GetKey(char *allowed, char *orig_allowed, gboolean AllowOther,
+static int GetKey(const char *orig_allowed, gboolean AllowOther,
                   gboolean PrintAllowed, gboolean ExpandOut);
 static void clear_bottom(void), clear_screen(void);
 static void clear_line(int line), clear_exceptfor(int skip);
@@ -347,7 +348,7 @@ void display_intro(void)
                     "possible (and stay alive)!"));
   mvaddcentstr(8, _("You have one month of game time to make your fortune."));
 
-  g_string_printf(text, _("Version %-8s Copyright (C) 1998-2020  Ben Webb "
+  g_string_printf(text, _("Version %-8s Copyright (C) 1998-2022  Ben Webb "
                            "benwebb@users.sf.net"), VERSION);
   mvaddcentstr(10, text->str);
   g_string_assign(text, _("dopewars is released under the GNU "
@@ -517,7 +518,7 @@ static gboolean SelectServerFromMetaServer(Player *Play, GString *errstr)
        if you translate them, keep the keys in the same order (N>ext,
        P>revious, S>elect) as they are here, otherwise they'll do the
        wrong things. */
-    c = GetKey(_("NPS"), "NPS", FALSE, FALSE, FALSE);
+    c = GetKey(N_("NPS"), FALSE, FALSE, FALSE);
     switch (c) {
     case 'S':
       AssignName(&ServerName, ThisServer->Name);
@@ -735,7 +736,7 @@ static gboolean ConnectToServer(Player *Play)
 
       /* Translate these 4 keys in line with the above options, keeping
          the order the same (C>onnect, L>ist, Q>uit, P>lay single-player) */
-      c = GetKey(_("CLQP"), "CLQP", FALSE, FALSE, FALSE);
+      c = GetKey(N_("CLQP"), FALSE, FALSE, FALSE);
       switch (c) {
       case 'Q':
         g_string_free(errstr, TRUE);
@@ -1033,7 +1034,7 @@ static void GiveErrand(Player *Play)
   /* Translate these 5 keys to match the above options, keeping the
      original order the same (S>py, T>ip off, G>et stuffed, C>ontact spy,
      N>o errand) */
-  c = GetKey(_("STGCN"), "STGCN", TRUE, FALSE, FALSE);
+  c = GetKey(N_("STGCN"), TRUE, FALSE, FALSE);
 
   if (Play->Bitches.Carried > 0 || c == 'C')
     switch (c) {
@@ -1056,7 +1057,7 @@ static void GiveErrand(Player *Play)
       /* The two keys that are valid for answering Yes/No - if you
          translate them, keep them in the same order - i.e. "Yes" before
          "No" */
-      c = GetKey(_("YN"), "YN", FALSE, TRUE, FALSE);
+      c = GetKey(N_("YN"), FALSE, TRUE, FALSE);
 
       if (c == 'Y')
         SendClientMessage(Play, C_NONE, C_SACKBITCH, NULL, NULL);
@@ -1079,7 +1080,7 @@ static int want_to_quit(void)
   attrset(PromptAttr);
   mvaddstr(get_prompt_line(), 1, _("Are you sure you want to quit? "));
   attrset(TextAttr);
-  return (GetKey(_("YN"), "YN", FALSE, TRUE, FALSE) != 'N');
+  return (GetKey(N_("YN"), FALSE, TRUE, FALSE) != 'N');
 }
 
 /* 
@@ -1241,7 +1242,7 @@ void HandleClientMessage(char *Message, Player *Play)
     wrd = GetNextWord(&pt, "");
     PrintMessage(pt);
     addch(' ');
-    i = GetKey(_(wrd), wrd, FALSE, TRUE, TRUE);
+    i = GetKey(wrd, FALSE, TRUE, TRUE);
     wrd = g_strdup_printf("%c", i);
     SendClientMessage(Play, C_NONE, C_ANSWER,
                       From == &Noone ? NULL : From, wrd);
@@ -1492,7 +1493,7 @@ void GunShop(Player *Play)
        the order (B>uy, S>ell, L>eave) the same - you can change the
        wording of the prompt, but if you change the order in this key
        list, the keys will do the wrong things! */
-    action = GetKey(_("BSL"), "BSL", FALSE, FALSE, FALSE);
+    action = GetKey(N_("BSL"), FALSE, FALSE, FALSE);
     if (action == 'S')
       SellGun(Play);
     else if (action == 'B')
@@ -1557,7 +1558,7 @@ void Bank(Player *Play)
 
     /* Make sure you keep the order the same if you translate these keys!
        (D>eposit, W>ithdraw, L>eave) */
-    action = GetKey(_("DWL"), "DWL", FALSE, FALSE, FALSE);
+    action = GetKey(N_("DWL"), FALSE, FALSE, FALSE);
 
     if (action == 'D' || action == 'W') {
       /* Prompt for putting money in or taking money out of the bank */
@@ -1589,23 +1590,39 @@ void Bank(Player *Play)
   } while (action != 'L' && money != 0);
 }
 
+/* Output a single Unicode character */
+static void addunich(gunichar ch, int attr)
+{
+  if (LocaleIsUTF8) {
+    char utf8buf[20];
+    gint utf8len = g_unichar_to_utf8(ch, utf8buf);
+    utf8buf[utf8len] = '\0';
+    attrset(attr);
+    addstr(utf8buf);
+  } else {
+    addch((guchar)ch | attr);
+  }
+}
+
+#define MAX_GET_KEY 30
+
 /* 
  * Waits for keyboard input; will only accept a key listed in the
- * "allowed" string. This string may have been translated; thus
- * the "orig_allowed" string contains the untranslated keys.
+ * translated form of the "orig_allowed" string.
  * Returns the untranslated key corresponding to the key pressed
- * (e.g. if allowed[2] is pressed, orig_allowed[2] is returned)
+ * (e.g. if translated(orig_allowed[2]) is pressed, orig_allowed[2] is returned)
  * Case insensitive. If "AllowOther" is TRUE, keys other than the
  * given selection are allowed, and cause a zero return value.
  * If "PrintAllowed" is TRUE, the allowed keys are printed after
  * the prompt. If "ExpandOut" is also TRUE, the full words for
  * the commands, rather than just their first letters, are displayed.
  */
-int GetKey(char *allowed, char *orig_allowed, gboolean AllowOther,
+int GetKey(const char *orig_allowed, gboolean AllowOther,
            gboolean PrintAllowed, gboolean ExpandOut)
 {
-  int ch;
-  guint AllowInd, WordInd, i;
+  int ch, i, num_allowed;
+  guint AllowInd, WordInd;
+  gunichar allowed[MAX_GET_KEY];
 
   /* Expansions of the single-letter keypresses for the benefit of the
      user. i.e. "Yes" is printed for the key "Y" etc. You should indicate
@@ -1617,15 +1634,36 @@ int GetKey(char *allowed, char *orig_allowed, gboolean AllowOther,
   guint numWords = sizeof(Words) / sizeof(Words[0]);
   gchar *trWord;
 
+  /* Translate allowed keys
+   * Note that allowed is in the locale encoding, usually UTF-8, while
+   * orig_allowed is plain ASCII */
+  char *allowed_str = _(orig_allowed);
+
+  num_allowed = strlen(orig_allowed);
+  assert(num_allowed <= MAX_GET_KEY);
+  assert(strcharlen(allowed_str) == num_allowed);
+
+  if (num_allowed == 0)
+    return 0;
+
+  /* Get Unicode allowed keys */
+  if (LocaleIsUTF8) {
+    const char *pt;
+    for (pt = allowed_str, i = 0; pt && *pt; pt = g_utf8_next_char(pt), ++i) {
+      allowed[i] = g_utf8_get_char(pt);
+    }
+  } else {
+    for (i = 0; i < num_allowed; ++i) {
+      allowed[i] = (guchar)allowed_str[i];
+    }
+  }
+
   curs_set(1);
   ch = '\0';
 
-  if (!allowed || strlen(allowed) == 0)
-    return 0;
-
   if (PrintAllowed) {
     addch('[' | TextAttr);
-    for (AllowInd = 0; AllowInd < strlen(allowed); AllowInd++) {
+    for (AllowInd = 0; AllowInd < num_allowed; AllowInd++) {
       if (AllowInd > 0)
         addch('/' | TextAttr);
       WordInd = 0;
@@ -1634,11 +1672,14 @@ int GetKey(char *allowed, char *orig_allowed, gboolean AllowOther,
         WordInd++;
 
       if (ExpandOut && WordInd < numWords) {
-        trWord = _(Words[WordInd]);
-        for (i = 2; i < strlen(trWord); i++)
-          addch((guchar)trWord[i] | TextAttr);
-      } else
-        addch((guchar)allowed[AllowInd] | TextAttr);
+        trWord = strchr(_(Words[WordInd]), ':');
+        /* Print everything after the colon */
+        if (*trWord) trWord++;
+        attrset(TextAttr);
+        addstr(trWord);
+      } else {
+        addunich(allowed[AllowInd], TextAttr);
+      }
     }
     addch(']' | TextAttr);
     addch(' ' | TextAttr);
@@ -1646,7 +1687,7 @@ int GetKey(char *allowed, char *orig_allowed, gboolean AllowOther,
 
   do {
     ch = bgetch();
-    ch = toupper(ch);
+    ch = LocaleIsUTF8 ? g_unichar_toupper(ch) : toupper(ch);
     /* Handle scrolling of message window */
     if (ch == '-') {
       scroll_msg_area_up();
@@ -1655,9 +1696,9 @@ int GetKey(char *allowed, char *orig_allowed, gboolean AllowOther,
       scroll_msg_area_down();
       continue;
     }
-    for (AllowInd = 0; AllowInd < strlen(allowed); AllowInd++) {
+    for (AllowInd = 0; AllowInd < num_allowed; AllowInd++) {
       if (allowed[AllowInd] == ch) {
-        addch((guint)ch | TextAttr);
+        addunich(allowed[AllowInd], TextAttr);
         curs_set(0);
         return orig_allowed[AllowInd];
       }
@@ -2231,7 +2272,8 @@ Player *ListPlayers(Player *Play, gboolean Select, char *Prompt)
 char *nice_input(char *prompt, int sy, int sx, gboolean digitsonly,
                  char *displaystr, char passwdchar)
 {
-  int i, c, x;
+  int ibyte, ichar, x;
+  gunichar c;
   gboolean DecimalPoint, Suffix;
   GString *text;
   gchar *ReturnString;
@@ -2248,47 +2290,72 @@ char *nice_input(char *prompt, int sy, int sx, gboolean digitsonly,
   attrset(TextAttr);
   if (displaystr) {
     if (passwdchar) {
-      for (i = strlen(displaystr); i; i--)
+      int i;
+      for (i = strcharlen(displaystr); i; i--)
         addch((guint)passwdchar);
     } else {
       addstr(displaystr);
     }
-    i = strlen(displaystr);
+    ibyte = strlen(displaystr);
+    ichar = strcharlen(displaystr);
     text = g_string_new(displaystr);
   } else {
-    i = 0;
+    ibyte = ichar = 0;
     text = g_string_new("");
   }
 
   curs_set(1);
   do {
-    move(sy + (x + i) / Width, (x + i) % Width);
+    move(sy + (x + ichar) / Width, (x + ichar) % Width);
     c = bgetch();
-    if ((c == 8 || c == KEY_BACKSPACE || c == 127) && i > 0) {
-      move(sy + (x + i - 1) / Width, (x + i - 1) % Width);
+    if ((c == 8 || c == KEY_BACKSPACE || c == 127) && ichar > 0) {
+      move(sy + (x + ichar - 1) / Width, (x + ichar - 1) % Width);
       addch(' ');
-      i--;
-      if (DecimalPoint && text->str[i] == '.')
+      ichar--;
+      if (LocaleIsUTF8) {
+        char *p = g_utf8_find_prev_char(text->str, text->str+ibyte);
+        assert(p);
+        ibyte = p - text->str;
+      } else {
+        ibyte--;
+      }
+      if (DecimalPoint && text->str[ibyte] == '.')
         DecimalPoint = FALSE;
       if (Suffix)
         Suffix = FALSE;
-      g_string_truncate(text, i);
+      g_string_truncate(text, ibyte);
     } else if (!Suffix) {
       if ((digitsonly && c >= '0' && c <= '9') ||
-          (!digitsonly && c >= 32 && c != '^' && c < 127)) {
-        g_string_append_c(text, c);
-        i++;
-        addch((guint)passwdchar ? passwdchar : c);
+          (!digitsonly && c >= 32 && c != '^' && c != 127)) {
+	if (LocaleIsUTF8) {
+          char utf8buf[20];
+          gint utf8len = g_unichar_to_utf8(c, utf8buf);
+          utf8buf[utf8len] = '\0';
+          ibyte += utf8len;
+          g_string_append(text, utf8buf);
+          if (passwdchar) {
+            addch((guint)passwdchar);
+          } else {
+            addstr(utf8buf);
+          }
+        } else {
+          g_string_append_c(text, c);
+          ibyte++;
+          addch((guint)passwdchar ? passwdchar : c);
+        }
+        ichar++;
       } else if (digitsonly && (c == '.' || c == ',') && !DecimalPoint) {
         g_string_append_c(text, '.');
-        i++;
+        ibyte++;
+        ichar++;
         addch((guint)passwdchar ? passwdchar : c);
         DecimalPoint = TRUE;
       } else if (digitsonly
                  && (c == 'M' || c == 'm' || c == 'k' || c == 'K')
                  && !Suffix) {
         g_string_append_c(text, c);
-        i++;
+        ibyte++;
+        ichar++;
         addch((guint)passwdchar ? passwdchar : c);
         Suffix = TRUE;
       }
@@ -2563,13 +2630,13 @@ static void Curses_DoGame(Player *Play)
       /* N.B. You must keep the order of these keys the same as the
          original when you translate (B>uy, S>ell, D>rop, T>alk, P>age,
          L>ist, G>ive errand, F>ight, J>et, Q>uit) */
-      c = GetKey(_("BSDTPLGFJQ"), "BSDTPLGFJQ", TRUE, FALSE, FALSE);
+      c = GetKey(N_("BSDTPLGFJQ"), TRUE, FALSE, FALSE);
 
     } else if (DisplayMode == DM_FIGHT) {
       /* N.B. You must keep the order of these keys the same as the
          original when you translate (D>eal drugs, R>un, F>ight, S>tand,
          Q>uit) */
-      c = GetKey(_("DRFSQ"), "DRFSQ", TRUE, FALSE, FALSE);
+      c = GetKey(N_("DRFSQ"), TRUE, FALSE, FALSE);
 
     } else
       c = 0;
@@ -2603,7 +2670,7 @@ static void Curses_DoGame(Player *Play)
           mvaddstr(get_prompt_line() + 1, 20,
                    _("List what? P>layers or S>cores? "));
           /* P>layers, S>cores */
-          i = GetKey(_("PS"), "PS", TRUE, FALSE, FALSE);
+          i = GetKey(N_("PS"), TRUE, FALSE, FALSE);
           if (i == 'P') {
             ListPlayers(Play, FALSE, NULL);
           } else if (i == 'S') {
@@ -2763,7 +2830,7 @@ void CursesLoop(struct CMDLINE *cmdline)
     RestoreConfig();
     attrset(TextAttr);
     mvaddstr(get_prompt_line() + 1, 20, _("Play again? "));
-    c = GetKey(_("YN"), "YN", TRUE, TRUE, FALSE);
+    c = GetKey(N_("YN"), TRUE, TRUE, FALSE);
   } while (c == 'Y');
   FirstClient = RemovePlayer(Play, FirstClient);
   end_curses();
