@@ -1,6 +1,6 @@
 /************************************************************************
  * newgamedia.c   New game dialog                                       *
- * Copyright (C)  1998-2020  Ben Webb                                   *
+ * Copyright (C)  1998-2022  Ben Webb                                   *
  *                Email: benwebb@users.sf.net                           *
  *                WWW: https://dopewars.sourceforge.io/                 *
  *                                                                      *
@@ -488,7 +488,7 @@ void NewGameDialog(Player *play)
 #endif
 
 #ifdef NETWORKING
-  GtkWidget *clist, *scrollwin, *table, *hbbox, *defbutton;
+  GtkWidget *clist, *scrollwin, *grid, *hbbox, *defbutton;
   GtkTreeSelection *treesel;
   gchar *ServerEntry, *text;
   gboolean UpdateMeta = FALSE;
@@ -550,15 +550,14 @@ void NewGameDialog(Player *play)
 #ifdef NETWORKING
   vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 7);
   gtk_container_set_border_width(GTK_CONTAINER(vbox2), 8);
-  table = gtk_table_new(2, 2, FALSE);
-  gtk_table_set_row_spacings(GTK_TABLE(table), 4);
-  gtk_table_set_col_spacings(GTK_TABLE(table), 4);
+  grid = dp_gtk_grid_new(2, 2, FALSE);
+  gtk_grid_set_row_spacing(GTK_GRID(grid), 4);
+  gtk_grid_set_column_spacing(GTK_GRID(grid), 4);
 
   /* Prompt for hostname to connect to in GTK+ new game dialog */
   label = gtk_label_new(_("Host name"));
 
-  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
-                   GTK_SHRINK, GTK_SHRINK, 0, 0);
+  dp_gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1, FALSE);
   entry = stgam.hostname = gtk_entry_new();
 
   ServerEntry = "localhost";
@@ -573,17 +572,16 @@ void NewGameDialog(Player *play)
     ServerEntry = ServerName;
 
   gtk_entry_set_text(GTK_ENTRY(entry), ServerEntry);
-  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
+  dp_gtk_grid_attach(GTK_GRID(grid), entry, 1, 0, 1, 1, TRUE);
   label = gtk_label_new(_("Port"));
-  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-                   GTK_SHRINK, GTK_SHRINK, 0, 0);
+  dp_gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1, FALSE);
   entry = stgam.port = gtk_entry_new();
   text = g_strdup_printf("%d", Port);
   gtk_entry_set_text(GTK_ENTRY(entry), text);
   g_free(text);
-  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 1, 2);
+  dp_gtk_grid_attach(GTK_GRID(grid), entry, 1, 1, 1, 1, TRUE);
 
-  gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox2), grid, FALSE, FALSE, 0);
 
   button = gtk_button_new_with_label("");
   /* Button to connect to a named dopewars server */
@@ -635,7 +633,7 @@ void NewGameDialog(Player *play)
   hbbox = my_hbbox_new();
 
   /* Button to update metaserver information */
-  button = NewStockButton(GTK_STOCK_REFRESH, accel_group);
+  button = gtk_button_new_with_mnemonic(_("_Refresh"));
   g_signal_connect(G_OBJECT(button), "clicked",
                    G_CALLBACK(UpdateMetaServerList), NULL);
   my_gtk_box_pack_start_defaults(GTK_BOX(hbbox), button);
@@ -719,7 +717,7 @@ static void DestroySocksAuth(GtkWidget *window, gpointer data)
 
 static void SocksAuthDialog(NetworkBuffer *netbuf, gpointer data)
 {
-  GtkWidget *window, *button, *hsep, *vbox, *label, *entry, *table, *hbbox;
+  GtkWidget *window, *button, *hsep, *vbox, *label, *entry, *grid, *hbbox;
   GtkAccelGroup *accel_group;
 
   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -742,19 +740,19 @@ static void SocksAuthDialog(NetworkBuffer *netbuf, gpointer data)
 
   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 7);
 
-  table = gtk_table_new(2, 2, FALSE);
-  gtk_table_set_row_spacings(GTK_TABLE(table), 10);
-  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+  grid = dp_gtk_grid_new(2, 2, FALSE);
+  gtk_grid_set_row_spacing(GTK_GRID(grid), 10);
+  gtk_grid_set_column_spacing(GTK_GRID(grid), 5);
 
   label = gtk_label_new("User name:");
-  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
+  dp_gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1, FALSE);
 
   entry = gtk_entry_new();
   g_object_set_data(G_OBJECT(window), "username", (gpointer)entry);
-  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
+  dp_gtk_grid_attach(GTK_GRID(grid), entry, 1, 0, 1, 1, TRUE);
 
   label = gtk_label_new("Password:");
-  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
+  dp_gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1, FALSE);
 
   entry = gtk_entry_new();
   g_object_set_data(G_OBJECT(window), "password", (gpointer)entry);
@@ -764,21 +762,21 @@ static void SocksAuthDialog(NetworkBuffer *netbuf, gpointer data)
   gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
 #endif
 
-  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 1, 2);
+  dp_gtk_grid_attach(GTK_GRID(grid), entry, 1, 1, 1, 1, TRUE);
 
-  gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), grid, TRUE, TRUE, 0);
 
   hsep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
   gtk_box_pack_start(GTK_BOX(vbox), hsep, FALSE, FALSE, 0);
 
   hbbox = my_hbbox_new();
 
-  button = NewStockButton(GTK_STOCK_OK, accel_group);
+  button = gtk_button_new_with_mnemonic(_("_OK"));
   g_signal_connect(G_OBJECT(button), "clicked",
                    G_CALLBACK(OKSocksAuth), (gpointer)window);
   my_gtk_box_pack_start_defaults(GTK_BOX(hbbox), button);
 
-  button = NewStockButton(GTK_STOCK_CANCEL, accel_group);
+  button = gtk_button_new_with_mnemonic(_("_Cancel"));
   g_signal_connect_swapped(G_OBJECT(button), "clicked",
                            G_CALLBACK(gtk_widget_destroy),
                            G_OBJECT(window));
